package com.clx.performance.service.impl;


import com.clx.performance.dao.OrderChildDao;
import com.clx.performance.dao.OrderChildImageDao;
import com.clx.performance.dao.OrderChildPoundAuditDao;
import com.clx.performance.dao.OrderChildPoundLogDao;
import com.clx.performance.enums.OrderChildEnum;
import com.clx.performance.enums.PerformanceResultEnum;
import com.clx.performance.model.OrderChild;
import com.clx.performance.model.OrderChildImage;
import com.clx.performance.model.OrderChildPoundAudit;
import com.clx.performance.model.OrderChildPoundLog;
import com.clx.performance.param.pc.PoundAuditParam;
import com.clx.performance.service.OrderChildPoundAuditService;
import com.clx.performance.vo.pc.OrderChildPoundAuditDetailVO;
import com.msl.common.enums.ResultCodeEnum;
import com.msl.user.data.UserSessionData;
import com.msl.user.utils.TokenUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author liruixin
 * Date 2023-09-19
 * Time 13:25
 */
@Slf4j
@Service
@AllArgsConstructor
public class OrderChildPoundAuditServiceImpl  implements OrderChildPoundAuditService {

    @Autowired
    OrderChildImageDao orderChildImageDao;
    @Autowired
    OrderChildPoundAuditDao orderChildPoundAuditDao;
    @Autowired
    OrderChildPoundLogDao orderChildPoundLogDao;
    @Autowired
    OrderChildDao orderChildDao;


    @Override
    public OrderChildPoundAuditDetailVO getPoundAuditDetail(String childNo) {
        OrderChildPoundAudit poundAuditDetail = orderChildPoundAuditDao.getPoundAuditDetail(childNo).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);
        List<OrderChildImage> loadImages = orderChildImageDao.getImages(childNo, OrderChildImage.Type.LOAD.getCode()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);
        List<OrderChildImage> unloadImages = orderChildImageDao.getImages(childNo, OrderChildImage.Type.UNLOAD.getCode()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);
        OrderChildPoundAuditDetailVO vo = new OrderChildPoundAuditDetailVO();
        vo.setChildNo(childNo);
        vo.setLoadImages(loadImages.stream().map(OrderChildImage::getImage).collect(Collectors.toList()));
        vo.setUnloadImages(unloadImages.stream().map(OrderChildImage::getImage).collect(Collectors.toList()));
        vo.setLoadNet(poundAuditDetail.getLoadNet());
        vo.setUnloadNet(poundAuditDetail.getUnloadNet());
        vo.setStatus(poundAuditDetail.getStatus());
        vo.setRemark(poundAuditDetail.getRemark());
        vo.setRejectType(poundAuditDetail.getRejectType());
        return vo;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePoundAudit(PoundAuditParam param) {

        OrderChild orderChild = orderChildDao.getByChildNo(param.getChildNo()).orElseThrow(ResultCodeEnum.FAIL);

        OrderChildPoundAudit poundAuditDetail = orderChildPoundAuditDao.getPoundAuditDetail(param.getChildNo()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);
        poundAuditDetail.setStatus(param.getStatus());
        orderChild.setStatus(OrderChildEnum.Status.UNSETTLED.getCode());
        if(Objects.equals(param.getStatus(),OrderChildPoundAudit.AuditStatus.FAIL)){
            poundAuditDetail.setStatus(param.getStatus());
            poundAuditDetail.setRejectType(param.getRejectType());
            poundAuditDetail.setRemark(param.getRemark());
            orderChild.setStatus(OrderChildEnum.Status.POUND_AUDIT_FAIL.getCode());
        }
        if(Objects.nonNull(param.getLoadNet())){
            poundAuditDetail.setLoadNet(param.getLoadNet());
        }
        if(Objects.nonNull(param.getUnloadNet())){
            poundAuditDetail.setUnloadNet(param.getUnloadNet());
        }
        orderChildPoundAuditDao.saveEntity(poundAuditDetail);

        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        OrderChildPoundLog poundLog = OrderChildPoundLog.builder().childNo(param.getChildNo()).status(param.getStatus()).createType(OrderChildPoundLog.CreateType.CARRIER.getCode())
                .createBy(loginUserInfo.getUserNo()).createName(loginUserInfo.getUserName()).build();

        orderChildPoundLogDao.saveEntity(poundLog);

        if(Objects.nonNull(param.getLoadNet())){
            orderChild.setLoadNet(param.getLoadNet());
        }
        if(Objects.nonNull(param.getUnloadNet())){
            orderChild.setUnloadNet(param.getUnloadNet());
        }
        orderChild.setPoundStatus(param.getStatus());
        orderChildDao.updatePoundAuditStatus(orderChild);

    }
}
